MishaTools_LoadScripts.RequireOnce(
['MishaTools/Helpers.js',
'MishaTools/Canvas/Rect.js',
'MishaTools/Observable.js'],
function() {
	MishaTools_Canvas = Class.create(MishaTools_Observable,{
	
		initialize:function($super,canvas,nointeraction) {
			$super();
			
			this.context = canvas.getContext('2d');
			
			this.stage = new MishaTools_Canvas_Rect('stage');
			
			this.stage.canvas = this;
			
			this.stage.setOptions({
				size:$C(this.context.canvas.width,this.context.canvas.height)
			});

			this.rendered = false;
			this.frozen = false;
			this.hits = [];
			this.focus = [];
			this.tabindex = 0;
			
			if(!nointeraction) {
				if(IsMobileBrowser()) {
					document.observe('touchstart',function(e) {
						this.down(e);
					}.bindAsEventListener(this));
					
					document.observe('touchmove',function(e) {
						this.move(e);
					}.bindAsEventListener(this));
					
					document.observe('touchend',function(e) {
						this.up(e);
					}.bindAsEventListener(this));
				} else {
					document.observe('mousemove',function(e) {
						this.move(e);
					}.bindAsEventListener(this));
					
					document.observe('mousedown',function(e) {
						this.down(e);
					}.bindAsEventListener(this));
					
					document.observe('mouseup',function(e) {
						this.up(e);
					}.bindAsEventListener(this));
				}
				
				canvas.observe('DOMMouseScroll',function(e) {
					this.hits.invoke('fire','mousescroll',{event:e,scroll:e.detail,vertical:e.axis == e.VERTICAL_AXIS});
					Event.stop(e);
				}.bindAsEventListener(this));
				
				canvas.observe('mousewheel',function(e) {
					if(e.wheelDeltaX != 0) this.hits.invoke('fire','mousescroll',{event:e,scroll:Math.round(e.wheelDeltaX*-1/6),vertical:false});
					if(e.wheelDeltaY != 0) this.hits.invoke('fire','mousescroll',{event:e,scroll:Math.round(e.wheelDeltaY*-1/6),vertical:true});
					
					Event.stop(e);
				}.bindAsEventListener(this));
				
				document.observe('keydown',function(e) {
					if(e.keyCode == Event.KEY_TAB && this.focus.length > 0) {
						this.nextTabIndex();
						Event.stop(e);
					} else {
						this.focus.invoke('fire','keydown',{event:e});
					}
				}.bindAsEventListener(this));
				
				document.observe('keypress',function(e) {
					this.focus.invoke('fire','keypress',{event:e});
				}.bindAsEventListener(this));
			}
			
			RequestAnimFrame(this.drawFrame.bind(this));
		},
		
		down:function(e) {
			var coord = MishaTools_Canvas.DocumentToCanvasCoord(this.context.canvas,$C(e.pointer()));
			if(coord) {
				this.mousedown = true;
				var hits = this.findHits(coord);
				this.changeHits(hits);
				this.tabindex = hits.invoke('getOptions','taborder').max() || 0;
				this.changeFocus(hits);
				var clickcount = (this.lastmousetime && (Date.now() - this.lastmousetime) < 500 && this.lastmousedownpos.subtract(coord).abs().lessThan($C(10))) ? 1 : 0;
				if(clickcount) clickcount += this.clickcount || 1;
				else clickcount = 0;
				
				this.focus.invoke('fire','mousedown',{position:coord,event:e,count:clickcount});
				this.draglastmousepos = coord;
				this.lastmousedownpos = coord;
				this.lastmousetime = Date.now();
				this.clickcount = clickcount;
			} else this.changeFocus([]);
		},
		
		move:function(e) {
			var coord = MishaTools_Canvas.DocumentToCanvasCoord(this.context.canvas,$C(e.pointer()));
					
			if(coord) {
				if(this.mousedown) {
					this.hits.invoke('fire','mousedrag',{position:coord,lastposition:this.draglastmousepos,mousedownposition:this.lastmousedownpos,event:e});
					this.draglastmousepos = coord;
				} else {
					var hits = this.findHits(coord);
					this.changeHits(hits);
					hits.invoke('fire','mousemove',{position:coord, event:e});
				}
			}
		},
		
		up:function(e) {
			var coord = MishaTools_Canvas.DocumentToCanvasCoord(this.context.canvas,$C(e.pointer()));
			this.mousedown = false;
			this.hits.invoke('fire','mouseup',{position:coord, event:e});
			this.changeHits([]);
		},
		
		freeze:function() {
			if(this.frozen) return;
			
			this.frozen = true;
			this.fire('freeze');
		},
		
		unfreeze:function() {
			if(!this.frozen) return;
			
			this.frozen = false;
			this.fire('unfreeze');
		},
		
		resize:function(coord) {
			this.stage.setOptions({
				size:$C(coord.x,coord.y)
			});
			this.rendered = false;
			this.fire('resize');
		},
		
		addChild:function(child) {
			this.stage.addChild(child);
		},
		
		findHits:function(coord) {
			var findHits = function(obj,coord) {
				if(obj && obj.hitTest(coord)) {
					return [obj].concat(findHits(obj.children.findAll(function(obj) { return obj.hitTest(coord); }).last(),coord));
				} else return [];
			};
			
			return findHits(this.stage,coord);
		},
		
		changeFocus:function(focus) {
			var oldfocus = this.focus;
			ArrayDiff(oldfocus,focus).invoke('fire','unfocus');
			ArrayDiff(focus,oldfocus).invoke('fire','focus');
			this.focus = focus;
		},
		
		changeHits:function(hits) {
			var oldhits = this.hits;
			ArrayDiff(oldhits,hits).invoke('fire','mouseout');
			ArrayDiff(hits,oldhits).invoke('fire','mouseover');
			this.hits = hits;
		},
		
		render:function() {
			if(!this.rendered && !this.frozen) {
				MishaTools_Canvas.ResetContext(this.context,this.stage.getOptions('size'));
				var objs = this.getDrawingOrder();
				objs.invoke('draw');
				this.rendered = true;
			}
		},
		
		drawFrame:function() {
			this.render();
			RequestAnimFrame(this.drawFrame.bind(this));
		},
		
		getDrawingOrder:function() {
			return [this.stage].concat(this.stage.getDescendents());
		},
		
		nextTabIndex:function() {
			var index = this.tabindex+1;
			var desc = this.stage.getDescendents();
			
			var tabfocus = undefined;
			var newtabindex = undefined;
			for(var i=0; i<desc.length;i++) {
				var obj = desc[i];
				if(obj.getOptions('taborder') == index) {
					tabfocus = obj;
					newtabindex = index;
				}
			}
			
			if(!tabfocus) tabfocus = this.stage;
			
			this.tabindex = newtabindex || 0;
			
			tabfocus.tab();
			tabfocus.fire('tab');
		}
	});
	
	Object.extend(MishaTools_Canvas,{
		
		SizeCanvas:function(canvas,size) {
			var sized = false;
			if(size.x != canvas.width) {
				canvas.width = size.x;
				sized = true;
			} 
			if(size.y != canvas.height) {
				canvas.height = size.y;
				sized = true;
			}
			return sized;
		},
		
		ResetContext:function(ctx,size) {
			var sized = false;
			if(size) sized = MishaTools_Canvas.SizeCanvas(ctx.canvas,size);
			if(!sized && size !== false) MishaTools_Canvas.ClearContext(ctx);
		},
		
		ClearContext:function(ctx) {
			ctx.clearRect(0,0,ctx.canvas.width,ctx.canvas.height);
		},
		
		DocumentToCanvasCoord:function(canvas,coord) {
			var origin = canvas.cumulativeOffset();
			var canvascoord = coord.subtract($C(origin.left,origin.top));
			return canvascoord;
		},
		
		CreateFullWindowCanvas:function() {
			var canvaselement = new Element('canvas',{style:'position:absolute; top:0px; left:0px; -moz-user-select: none; -webkit-user-select: none; -ms-user-select: none;',width:window.innerWidth,height:window.innerHeight});
			
			var canvas = new MishaTools_Canvas(canvaselement);
			
			window.onresize = function() {
				canvaselement.width = window.innerWidth;
				canvaselement.height = window.innerHeight;
				canvas.resize($C(window.innerWidth,window.innerHeight));
				canvas.render();
			};
			
			document.body.appendChild(canvaselement);
			
			return canvas;
		}
	});
});