MishaTools_LoadScripts.RequireOnce(
['MishaTools/Canvas.js',
'MishaTools/Canvas/Cache.js',
'MishaTools/Observable.js',
'MishaTools/Canvas/Coordinate/Rect.js',
'MishaTools/Canvas/Coordinate/Polar.js',
'MishaTools/Canvas/Transform.js',
'MishaTools/Dependency.js',
'MishaTools/Dependency/Time.js'],
function() {

	$O = function(obj,option) {
		return obj.getOptionDependency(option);
	};
	
	$PO = function(obj,option) {
	
		var parentoptiondependency = $D();
		
		var parentdependency = obj.getOptionDependency('parent');
		
		var updateparentoptiondependencylistener = function(e) {
			var parent = e.data.newvalue;
			if(parent) parentoptiondependency.change(function(dependency) { return dependency; },[parent.getOptionDependency(option)]);
			else parentoptiondependency.change(undefined);
		};
		
		parentoptiondependency.observe('active',function(e) {
			parentdependency.observe('evaluated',updateparentoptiondependencylistener);
		});
		
		parentoptiondependency.observe('inactive',parentdependency.selfDestructListener(function(e) {
			parentdependency.stopObserving('evaluated',updateparentoptiondependencylistener);
		}));
		
		return parentoptiondependency;
	};
	
	MishaTools_Canvas_Object = Class.create(MishaTools_Observable,{
		
		initialize:function($super,id) {
			$super();
			
			this.id = id; //unique
			this.options = {};
			this.children = [];
			this.parent = null;
			this.rendered = false;
			this.frozen = false;
			this.caching = true;
			this.renderimage = null;
			this.canvas = null;
			
			this.invalidaterenderingoptionchangelistener = this.invalidateRendering.bind(this);
			this.invalidatecanvasrenderingunfreezelistener = this.selfDestructListener(this.invalidateCanvasRendering.bind(this));
			
			this.setOptions({
				parent:null,
				origin:$C(0),
				angle:0,
				scale:$C(1),
				focus:false,
				taborder:null,
				draggable:false
			});
			
			this.setOptionDefinitions({
				transform:{
						func:function(parenttransform,origin,angle,scale) { return (parenttransform || MishaTools_Canvas_Transform.Identity()).translate(origin).rotate(angle).scale(scale); },
						dependencies:
						[$PO(this,'transform'),
						$O(this,'origin'),
						$O(this,'angle'),
						$O(this,'scale')],
						render:true
					}
			});
			
			this.observe('focus',function(e) {
				this.setOptions({focus:true});
			}.bind(this));
			
			this.observe('unfocus',function(e) {
				this.setOptions({focus:false});
			}.bind(this));
			
			var draggablelistener = function(e) {
				var lastcoord = this.globalToLocal(e.data.lastposition);
				var coord = this.globalToLocal(e.data.position);
				var deltacoord = coord.subtract(lastcoord);
				
				this.setOptions({
					origin:this.getOptions('origin').add(deltacoord)
				});
			}.bind(this);
			
			$O(this,'draggable').observe('changed',function(e) {
				if(e.data.newvalue) this.observe('mousedrag',draggablelistener);
				else this.stopObserving('mousedrag',draggablelistener);
			}.bind(this));
		},
		
		render:function(ctx) {
		
			if(this._render) {
				this.transform(ctx);
				ctx.beginPath();
				if(this._bounds) {
					this._bounds(ctx);
					ctx.clip();
					ctx.beginPath();
				}
				this._render(ctx);
			}
			
		},
		
		transform:function(ctx) {
			var transform = this.getOptions('transform');
			ctx.setTransform(transform.m11,transform.m21,transform.m12,transform.m22,transform.dx,transform.dy);
		},
		
		getStageCache:function() {
			return MishaTools_Canvas_Cache.GetFreeCache(this.canvas.stage.getOptions('size'));
		},
		
		draw:function() {
			if(this.caching) {
				if(!this.rendered) {
					if(this._render) {
						var cache = this.getStageCache();
						this.render(cache.context);
						this.renderimage = cache.save();
						this.canvas.context.drawImage(cache.context.canvas,0,0);
						cache.release();
					} else {
						this.renderimage = null;
					}
					
					this.rendered = true;
				} else if(this.renderimage) {
					var cache = this.getStageCache();
					cache.context.putImageData(this.renderimage,0,0);
					this.canvas.context.drawImage(cache.context.canvas,0,0);
					cache.release();
				}
			} else {
				this.canvas.context.save();
				this.render(this.canvas.context);
				this.canvas.context.restore();
			}
		},
		
		hitTest:function(coord) {
			if(this._bounds) {
				var cache = this.getStageCache();
				ctx = cache.context;
				
				this.transform(ctx);
				
				ctx.beginPath();
				this._bounds(ctx);
				
				var test = ctx.isPointInPath(coord.x,coord.y);
				cache.release();
				return test;
			}
			
			return false;
		},
		
		addChild:function(child) {
			
			if(child.parent) child.parent.removeChild(child);
			
			child.parent = this;
			child.setOptions({parent:this});
			
			child.setCanvas(this.canvas);
			
			this.children.push(child);
			
			this.invalidateCanvasRendering();
		},
		
		removeChild:function(child) {
			
			if(child.parent != this) return;
			
			if(child.getOptions('focus')) this.focus();
			
			this.children = this.children.reject(function(i) { return i == child; });
			
			child.parent = null;
			child.setOptions({parent:null});
			child.setCanvas(null);
			
			this.invalidateCanvasRendering();
		},
		
		getChild:function() {
			var args = $A(arguments);
			
			var getchild = function(obj,id) {
				return obj.children.find(function(child) { return child.id == id; })
			};
			
			var child = args.inject(this,function(agg,id) {
				var child = getchild(agg,id);
				if(Object.isUndefined(child)) throw $break;
				return child;
			});
			
			return child == this ? undefined : child;
		},
		
		getDescendents:function() {
		
			var getDescendents = function(obj) {
				return obj.children.collect(function(child) {
					return [child].concat(getDescendents(child));
				});
			};
			
			return getDescendents(this).flatten();
		},
		
		getAncestors:function() {
			
			var getAncestors = function(obj) {
				return obj.parent ? [obj].concat(getAncestors(obj.parent)) : [obj];
			};
			
			return this.parent ? getAncestors(this.parent) : [];
		},
		
		getSiblings:function() {
			return this.parent ? this.parent.children.reject(function(i) { return i == this; },this) : [];
		},

		getOptions:function() {
			var optionnames = $A(arguments);
			var options = optionnames.collect(function(option) {
				return this.options[option] && this.options[option].value;
			},this);
			
			if(options.length == 1) return options[0];
			else {
				return options.inject({},function(agg,option) {
					agg[option] = optionnames[option];
					return agg;
				});
			}
		},
		
		getOptionDependency:function(option) {
			if(!this.options[option]) {
				var dependency = $D();
				this.options[option] = dependency;
				return dependency;
			} else return this.options[option];
		},
		
		setFamilyOptions:function(sourceoptions) {
			var frozen = this.frozen;
			if(!frozen) this.freeze();
			
			Object.keys(sourceoptions).each(function(key) {
				var child = this.getChild(key);
				
				if(!Object.isUndefined(child)) {
					child.setOptions(sourceoptions[key]);	
				} else {
					var val = sourceoptions[key];
					this.getOptionDependency(key).change(function() { return val; });
				}
			},this);
			
			if(!frozen) this.unfreeze();
		},
		
		setOptions:function(sourceoptions) {
			var frozen = this.frozen;
			if(!frozen) this.freeze();
			
			Object.keys(sourceoptions).each(function(key) {
				var val = sourceoptions[key];
				this.getOptionDependency(key).change(function() { return val; });
			},this);
			
			if(!frozen) this.unfreeze();
		},
		
		setOptionDefinitions:function(sourceoptions) {
			
			var frozen = this.frozen;
			if(!frozen) this.freeze();
			
			Object.keys(sourceoptions).each(function(key) {
				var definition = sourceoptions[key];
				
				if(Object.isFunction(definition.equalityfunc)) {
					this.getOptionDependency(key).equalityfunc = definition.equalityfunc;
				}
				
				if(!Object.isUndefined(definition.value)) {
					this.getOptionDependency(key).change(function() { return definition.value; });
				} else if(definition.dependsOn) {
					this.getOptionDependency(key).change(function(dependency) { return dependency; },[definition.dependsOn]);
				} else if(definition.func) {
					var dependencies = definition.dependencies || this.getOptionDependency(key).dependencies;
					var dependency = this.getOptionDependency(key);
					
					if(definition.override === true) {
						//create a new dependency out of the existing dependency and pass it to the new dependency
						dependency.change(definition.func,[dependency.clone()].concat(dependencies));
					} else {
						dependency.change(definition.func,dependencies);
					}
				}
				
				if(definition.render === true) {
					this.getOptionDependency(key).observe('changed',this.invalidaterenderingoptionchangelistener);
				} else if(definition.render === false) {
					this.getOptionDependency(key).stopObserving('changed',this.invalidaterenderingoptionchangelistener);
				}
			},this);
			
			if(!frozen) this.unfreeze();
		},
		
		invalidateRendering:function() {
			this.rendered = false;
			this.invalidateCanvasRendering();
		},
		
		invalidateCanvasRendering:function() {
			if(this.canvas) {
				if(this.frozen) {
					this.observe('unfreeze',this.invalidatecanvasrenderingunfreezelistener);
				} else {
					this.canvas.rendered = false;
				}
			}
		},
		
		freeze:function() {
			if(this.frozen) return;
			
			this.frozen = true;
			this.fire('freeze');
		},
		
		unfreeze:function() {
			if(!this.frozen) return;
			
			this.frozen = false;
			this.fire('unfreeze');
		},
		
		focus:function() {
			if(this.canvas) {
				this.canvas.changeFocus([this].concat(this.getAncestors()).reverse());
			}
		},
		
		tab:function() {
			this.focus();
		},
		
		setCaching:function(caching) {
			//only turn this off if you are animating
			this.caching = caching;
		},
		
		setCanvas:function(canvas) {
			[this].concat(this.getDescendents()).each(function(obj) {
				obj.canvas = canvas;
			});
		},
		
		globalToLocal:function(coord) {
			var transform = this.getOptions('transform');
			return transform.globalToLocal(coord);
		},
		
		localToGlobal:function(coord) {
			var transform = this.getOptions('transform');
			return transform.localToGlobal(coord);
		},
		
		localToPage:function(coord) {
			if(this.canvas) {
				var canvasorigin = this.canvas.context.canvas.cumulativeOffset();
				return $C(canvasorigin.left,canvasorigin.top).add(this.localToGlobal($C(0)));
			} else return undefined;
		},
		
		center:function(offset) {
			if(!offset) offset = $C(0);
			
			this.setOptionDefinitions({
				origin:{
					func:function(parentsize,size) {
						return parentsize ? parentsize.divide(2).subtract(size.divide(2)).add(offset).round() : $C(0);
					},
					dependencies:[$PO(this,'size'),$O(this,'size')]
				}
			});
		}
	});
});