MishaTools_LoadScripts.RequireOnce(
['MishaTools/Canvas/Rect/Text.js',
'MishaTools/ClipboardAccess.js'],
function() {
	MishaTools_Canvas_Rect_Text_Interactive = Class.create(MishaTools_Canvas_Rect_Text,{
		
		initialize:function($super,id) {
			$super(id);
			
			MishaTools_ClipboardAccess.Create();
			
			this.setOptionDefinitions({
				focus:{render:true},
				editable:{value:true},
				multiline:{value:false},
				cursorstroke:{value:'rgba(128,128,128,1)',
							render:true},
				selectionfill:{value:'rgba(51,153,255,.5)',
							   render:true},
				_cursor:{value:0,render:true},
				_selection:{render:true},
				_anchor:{value:undefined},
				keyboard:{vaue:'text'} //xcoord
			});
			
			this.observe('keydown',function(e) {
				var keycode = e.data.event.keyCode;
				var shift = e.data.event.shiftKey;
				var metakey = e.data.event.metaKey;
				
				if(keycode == Event.KEY_BACKSPACE) {
					this.delete();
					Event.stop(e.data.event);
				} else if(keycode == Event.KEY_LEFT) {
					this.moveLeft(shift);
					Event.stop(e.data.event);
				} else if(keycode == Event.KEY_RIGHT) {
					this.moveRight(shift);
					Event.stop(e.data.event);
				} else if(keycode == Event.KEY_UP) {
					this.moveUp(shift);
					Event.stop(e.data.event);
				} else if(keycode == Event.KEY_DOWN) {
					this.moveDown(shift);
					Event.stop(e.data.event);
				} else if(keycode == 67 && metakey) { //c
					this.copy();
				} else if(keycode == 65 && metakey) { //a
					this.selectAll();
					Event.stop(e.data.event);
				} else if(keycode == 86 && metakey) { //v
					this.paste();
				} else if(keycode == 88 && metakey) { //x
					this.cut();
				}
			}.bind(this));
			
			this.observe('keypress',function(e) {
				var metakey = e.data.event.metaKey;
				var charcode = e.data.event.charCode;
				
				if(charcode > 0) {
					var charstr = String.fromCharCode(charcode);
					
					if(charstr == '\r' || charstr == '\n') {
						if(!this.getOptions('multiline')) return;
						else if(charstr == '\r') charstr = '\n';
					}
					
					if(charstr && !metakey) {
						this.write(charstr);
						Event.stop(e.data.event);
					}
				}
			}.bind(this));
			
			this.observe('mousedown',function(e) {
				var xoffset = this.getOptions('xoffset');
				var coord = this.globalToLocal(e.data.position).add($C(xoffset,0));
				this.click(coord,e.data.event.shiftKey,e.data.count);
			}.bind(this));
			
			this.observe('mouseover',function(e) {
				if(this.canvas) this.canvas.context.canvas.setStyle({cursor:'text'});
			}.bind(this));
			
			this.observe('mouseout',function(e) {
				if(this.canvas) this.canvas.context.canvas.setStyle({cursor:'auto'});
			}.bind(this));
			
			this.observe('mousedrag',function(e) {
				var coord = this.globalToLocal(e.data.position);
				var size = this.getOptions('size');
				
				if(coord.x >= 0 && coord.x <= size.x && coord.y >= 0 && coord.y <= size.y) {
					if(this.phantomdragpe) {
						this.phantomdragpe.stop()
						this.phantomdragpe = undefined;
					}
				} else {
					if(this.phantomdragpe) this.phantomdragpe.stop();
					this.phantomdragpe = new PeriodicalExecuter(function(pe) {
						this.drag(coord);
					}.bind(this),.1);
				}
				this.drag(coord);
				
			}.bind(this));
						
			this.observe('mouseup',function(e) {
				if(this.phantomdragpe) {
					this.phantomdragpe.stop()
					this.phantomdragpe = undefined;
				}
			}.bind(this));
			
			this.observe('mousescroll',function(e) {
				if(e.data.vertical) this.scroll(e.data.scroll);
			}.bind(this));
			
			$O(this,'_cursor').observe('changed',function(e) {
				this.setView(e.data.newvalue);
			}.bind(this));
			
			$O(this,'focus').observe('changed',function(e) {			
				//if it is mobile safari, position the dummy text box so the user can type
				if(IsMobileBrowser()) {
					var keyboard = this.getOptions('keyboard');
					MishaTools_Canvas_Rect_Text_Interactive.ActivateTextShim(this.localToPage($C(0)),keyboard);
					this.canvas.context.canvas.focus();
				}
			}.bind(this));
		},
		
		moveLeft:function(shift) {
			var cursor = this.getOptions('_cursor');
			var selection = this.getOptions('_selection');
				
			if(shift) {
				if(cursor > 0) {
					this.setOptions({
						_selection:Object.isUndefined(selection) ? cursor : selection,
						_cursor:cursor-1,
						_anchor:undefined
					});
				}
			} else {
				if(cursor > 0 || !Object.isUndefined(selection)) {
					this.setOptions({
						_selection:undefined,
						_cursor:!Object.isUndefined(selection) ? Math.min(cursor,selection) : cursor-1,
						_anchor:undefined
					});
				}
			}
		},
		
		moveRight:function(shift) {
			var cursor = this.getOptions('_cursor');
			var selection = this.getOptions('_selection');
			var text = this.getOptions('text');
			
			if(shift) {
				if(cursor < text.length) {
					this.setOptions({
						_selection:Object.isUndefined(selection) ? cursor : selection,
						_cursor:cursor+1,
						_anchor:undefined
					});
				}
			} else {
				if(cursor < text.length || !Object.isUndefined(selection)) {
					this.setOptions({
						_selection:undefined,
						_cursor:!Object.isUndefined(selection) ? Math.max(cursor,selection) : cursor+1,
						_anchor:undefined
					});
				}
			}
		},
		
		moveUp:function(shift) {
			var cursor = this.getOptions('_cursor');
			var selection = this.getOptions('_selection');
			var anchor = this.getOptions('_anchor');
			
			if(!shift && !Object.isUndefined(selection) && selection < cursor) cursor = selection;
			
			var cursorposition = this.getLocationAtPosition(cursor);
			
			if(cursorposition.y - 1 >= -1) {
				var newanchor = Object.isUndefined(anchor) ? this.getCoordAtLocation(cursorposition).x : anchor;
				var lineanchorxlocation = this.getXLocationAtXCoordInLine(newanchor,cursorposition.y);
				var newcursorlocation = cursorposition.x > lineanchorxlocation ? $C(lineanchorxlocation,cursorposition.y) : (cursorposition.y - 1 > -1 ? $C(this.getXLocationAtXCoordInLine(newanchor,cursorposition.y - 1),cursorposition.y - 1) : $C(0));
				
				this.setOptions({
					_selection: shift ? (Object.isUndefined(selection) ? cursor : selection) : undefined,
					_cursor:this.getPositionAtLocation(newcursorlocation),
					_anchor: newanchor
				});
			}
		},
		
		moveDown:function(shift) {
			var cursor = this.getOptions('_cursor');
			var selection = this.getOptions('_selection');
			var anchor = this.getOptions('_anchor');
			var text = this.getOptions('text');
			var lines = this.getOptions('_lines');
			
			if(!shift && !Object.isUndefined(selection) && selection > cursor) cursor = selection;
					
			var cursorposition = this.getLocationAtPosition(cursor);
			
			if(cursorposition.y + 1 <= lines.length) {
				var newanchor = Object.isUndefined(anchor) ? this.getCoordAtLocation(cursorposition).x : anchor;
				var lineanchorxlocation = this.getXLocationAtXCoordInLine(newanchor,cursorposition.y);
				var newcursorlocation = cursorposition.x < lineanchorxlocation ? $C(lineanchorxlocation,cursorposition.y) : (cursorposition.y + 1 < lines.length ? $C(this.getXLocationAtXCoordInLine(newanchor,cursorposition.y + 1),cursorposition.y + 1) : this.getLocationAtPosition(text.length));
				
				this.setOptions({
					_selection: shift ? (Object.isUndefined(selection) ? cursor : selection) : undefined,
					_cursor:this.getPositionAtLocation(newcursorlocation),
					_anchor: newanchor
				});
			}
		},
		
		delete:function() {
			var editable = this.getOptions('editable');
			var cursor = this.getOptions('_cursor');
			var selection = this.getOptions('_selection');
			var text = this.getOptions('text');
			
			if(!editable) return;
			
			if(!Object.isUndefined(selection)) {
				var leftbound = Math.min(selection,cursor);
				var rightbound = Math.max(selection,cursor);
				var length = rightbound - leftbound;
				
				this.setOptions({
					text:RemoveStringAtPos(text,rightbound,length),
					_cursor:leftbound,
					_selection:undefined,
					_anchor:undefined
				});
				
			} else {
				if(cursor - 1 >= 0) {
					this.setOptions({
						text:RemoveStringAtPos(text,cursor,1),
						_cursor:cursor-1,
						_anchor:undefined
					});
				}
			}
		},
		
		write:function(newtext) {
			
			var editable = this.getOptions('editable');
				
			if(!editable) return;
				
			if(newtext.length > 0) {
				
				var selection = this.getOptions('_selection');
				
				if(!Object.isUndefined(selection)) {
					this.delete();
				}
				
				var multiline = this.getOptions('multiline');
				var cursor = this.getOptions('_cursor');
				var text = this.getOptions('text');
				
				if(newtext.toArray().member('\n') && !multiline) {
					newtext = newtext.toArray().reject(function(c) { return c == '\n'; });
				}
				
				this.setOptions({
					text:InsertStringAtPos(text,cursor,newtext),
					_cursor:cursor+newtext.length,
					_anchor:undefined
				});
			}
		},
		
		selectAll:function() {
			var text = this.getOptions('text');
			
			this.setOptions({
				_cursor:text.length,
				_selection:0
			});
		},
		
		getSelection:function() {
			var cursor = this.getOptions('_cursor');
			var selection = this.getOptions('_selection');
			var text = this.getOptions('text');
			var obscure = this.getOptions('obscure');
			
			if(!Object.isUndefined(cursor) && !Object.isUndefined(selection) && text.length > 0 && !obscure) {
				return text.substring(selection,cursor);
			} else return '';
		},
		
		cut:function() {
			//requires command-x to work
			var selectedtext = this.getSelection();
			if(selectedtext) {
				MishaTools_ClipboardAccess.ClipboardAccess.interceptCut(selectedtext);
				this.delete();
			}
		},
		
		copy:function() {
			//requires command-c to work
			var selectedtext = this.getSelection();
			if(selectedtext) {
				MishaTools_ClipboardAccess.ClipboardAccess.interceptCopy(selectedtext);
			}
		},
		
		paste:function() {
			//requires command-v to work
			MishaTools_ClipboardAccess.ClipboardAccess.interceptPaste(function(pastetext) {
				if(pastetext) {
					var cursor = this.getOptions('_cursor');
					var selection = this.getOptions('_selection');
					var text = this.getOptions('text');
					
					if(!Object.isUndefined(selection)) {
						var leftbound = Math.min(selection,cursor);
						var rightbound = Math.max(selection,cursor);
						var length = rightbound - leftbound;
						
						this.setOptions({
							text:InsertStringAtPos(RemoveStringAtPos(text,rightbound,length),leftbound,pastetext),
							_cursor:leftbound+pastetext.length,
							_selection:undefined,
							_anchor:undefined
						});
					} else {
						this.setOptions({
							text:InsertStringAtPos(text,cursor,pastetext),
							_cursor:cursor+pastetext.length,
							_anchor:undefined
						});
					}
				}
			}.bind(this));
		},
		
		click:function(coord,shift,count) {
		
			if(IsMobileBrowser() && count >= 3) {
				this.selectAll();
				return;
			}
			
			var cursor = this.getOptions('_cursor');
			var selection = this.getOptions('_selection');
			var text = this.getOptions('text');
			
			var position = this.getPositionAtCoord(coord,true);
			var leftbound = undefined;
			var rightbound = undefined;
			
			var newposition = position;
			var newselection = undefined;
			
			if(text.length > 0) {
				if(count >= 2) {
					if(position == text.length && text.length > 0) position--;
					
					if(position < text.length) {
						if(IsAlphaNumeric(text[position])) {
							for(var i=position+1;i<text.length;i++) {
								if(!IsAlphaNumeric(text[i])) break;
							}
							for(var j=i;j<text.length;j++) {
								if(text[j] != ' ') break;
							}
							i = j;
						} else if(text[position] != ' ') {
							for(var i=position+1;i<text.length;i++) {
								if(IsAlphaNumeric(text[i]) || text[i] == ' ') break;
							}
							for(var j=i;j<text.length;j++) {
								if(text[j] != ' ') break;
							}
							i = j;
						} else {
							for(var i=position+1;i<text.length;i++) {
								if(text[i] != ' ') break;
							}
						}
						rightbound = i;
					}
	
					if(position > 0) {
						if(IsAlphaNumeric(text[position])) {
							for(var i=position-1;i>=0;i--) {
								if(!IsAlphaNumeric(text[i])) break;
							}
						} else if(text[position] != ' ') {
							for(var i=position-1;i>=0;i--) {
								if(IsAlphaNumeric(text[i]) || text[i] == ' ') break;
							}
						} else {
							for(var i=position-1;i>=0;i--) {
								if(text[i] != ' ') break;
							}
						}
						leftbound = i+1;
					} else if(!Object.isUndefined(rightbound)) leftbound = 0;
					
					if(shift) {
						newposition = selection > position ? leftbound : rightbound;
					} else {
						newposition = rightbound;
						newselection = leftbound;
					}
				}
				
				if(shift) {
					newselection = Object.isUndefined(selection) ? (Object.isUndefined(cursor) ? text.length : cursor) : selection;
					if((cursor > selection && position < selection) || (cursor < selection && position > selection)) newselection = cursor;
				}
			}
			
			this.setOptions({
				_cursor: newposition,
				_selection: newselection,
				_anchor:undefined
			});
		},
		
		drag:function(coord) {
			var size = this.getOptions('size');
			var xoffset = this.getOptions('xoffset');
			
			if(coord.x < 0) coord.x = 0;
			if(coord.x > size.x) coord.x = size.x;
			if(coord.y < 0) coord.y = 0;
			if(coord.y > size.y) coord.y = size.y;
			
			var coordlocation = this.getLocationAtCoord(coord.add($C(xoffset,0)),true);
			var position = this.getPositionAtLocation(coordlocation);
			var selection = this.getOptions('_selection');
			var renderlines = this.getOptions('_renderlines');
			var lineoffset = this.getOptions('lineoffset');
			var lines = this.getOptions('_lines');
			
			if(coord.x == 0) {
				position = coordlocation.x > 0 ? position - 1 : position;
			} else if(coord.x == size.x) {
				position = coordlocation.x < lines[coordlocation.y].length ? position + 1 : position;
			} else if(coord.y == 0) {
				position = coordlocation.y > 0 ? this.getPositionAtLocation($C(0,lineoffset-1)) : position;
			} else if(coord.y == size.y) {
				var lastline = lineoffset+renderlines.length-1;
				position = coordlocation.y < lines.length-1 ? this.getPositionAtLocation($C(lines[lastline+1].length,lastline+1)) : position;
			}
			
			this.setOptions({
				_cursor:position,
				_selection:Object.isUndefined(selection) ? position : selection
			});
		},
		
		setView:function(position) {
			var lineoffset = this.getOptions('lineoffset');
			var renderlines = this.getOptions('_renderlines');
			var xoffset = this.getOptions('xoffset');
			var size = this.getOptions('size');
			var align = this.getOptions('align');
			var newlineoffset = lineoffset;
			var newxoffset = xoffset;
			var textareasize = this.getOptions('_textareasize');
			
			var cursorlocation = this.getLocationAtPosition(position);
			
			var xcursor = this.getCoordAtLocation(cursorlocation).x;
			
			if(xcursor-xoffset < 0) newxoffset = xcursor;
			else if(xcursor-xoffset > size.x) newxoffset = xcursor-size.x;
			
			if((align == 'right' || align == 'center') && newxoffset + size.x >= textareasize.x) newxoffset = textareasize.x-size.x+1;
			
			var linenum = cursorlocation.y;
			if(linenum < lineoffset && lineoffset > 0) newlineoffset = linenum;
			else if(linenum > lineoffset+renderlines.length-1) newlineoffset = linenum-renderlines.length+1;
			
			this.setOptions({
				lineoffset:newlineoffset,
				xoffset:newxoffset
			});
		},
		
		scroll:function(scroll) {
			var lineoffset = this.getOptions('lineoffset');
			var lines = this.getOptions('_lines');
			var renderlines = this.getOptions('_renderlines');
			
			if((scroll < 0 && lineoffset+scroll >= 0) || (scroll > 0 && lineoffset+scroll <= lines.length-1-renderlines.length+1))
				this.setOptions({lineoffset:lineoffset+scroll});
		},
		
		_render:function($super,ctx) {

			var cursor = this.getOptions('_cursor');
			var focus = this.getOptions('focus');
			
			if(focus) {
				if(!Object.isUndefined(cursor)) {
					var fontsize = this.getOptions('fontsize');
					var selection = this.getOptions('_selection');
					var xoffset = this.getOptions('xoffset');
					
					ctx.save();
					
					if(!Object.isUndefined(selection) && selection != cursor) {
						var lines = this.getOptions('_lines');
						var renderlines = this.getOptions('_renderlines');
						var selectionfill = this.getOptions('selectionfill');
						var lineoffset = this.getOptions('lineoffset');
						var size = this.getOptions('size');
						
						ctx.fillStyle = selectionfill;
						
						var leftboundlocation = this.getLocationAtPosition(Math.min(selection,cursor));
						var rightboundlocation = this.getLocationAtPosition(Math.max(selection,cursor));
						
						var height = fontsize;
						
						for(var i=leftboundlocation.y;i<=rightboundlocation.y;i++) {
							if(i >= lineoffset && i < lineoffset+renderlines.length) {
								if(i == leftboundlocation.y) {
									var upperleftcoord = this.getCoordAtLocation(leftboundlocation);
									var endxcoord = (leftboundlocation.y == rightboundlocation.y) ? this.getCoordAtLocation($C(rightboundlocation.x,leftboundlocation.y)).x : size.x+xoffset;
									var width = endxcoord - upperleftcoord.x;
									
									ctx.rect(upperleftcoord.x-xoffset,upperleftcoord.y,width,height);
								} else if(i == rightboundlocation.y) {
									var upperleftcoord = this.getCoordAtLocation($C(0,rightboundlocation.y));
									var width = this.getCoordAtLocation(rightboundlocation).x;
									
									ctx.rect(-xoffset,upperleftcoord.y,width,height);
								} else if(i != leftboundlocation.y && i != rightboundlocation.y) {
									var upperleftcoord = this.getCoordAtLocation($C(0,i));
									ctx.rect(0,upperleftcoord.y,size.x,height);
								}
							}
						}
						
						ctx.fill();
					} else {
						var cursorcoord = this.getCoordAtPosition(cursor,true);
						if(cursorcoord) { 
							var cursorstroke = this.getOptions('cursorstroke');
							
							ctx.lineWidth = 1;
							ctx.strokeStyle = cursorstroke;
							ctx.moveTo(cursorcoord.x-xoffset,cursorcoord.y);
							ctx.lineTo(cursorcoord.x-xoffset,cursorcoord.y+fontsize);
							ctx.stroke();
						}
					}
					ctx.restore();
				}
			}
			
			$super(ctx);
		},
		
		getLineAtYCoord:function(ycoord) {
			var lines = this.getOptions('_lines');
			var lineoffset = this.getOptions('lineoffset');
			var fontsize = this.getOptions('fontsize');
			
			var linenum = undefined;
			for(var i=lineoffset;i<lines.length;i++) {
				var lowerbound = (i-lineoffset) * fontsize;
				var upperbound = (i-lineoffset + 1) * fontsize;
				if(ycoord >= lowerbound && ycoord < upperbound) {
					linenum = i;
					break;
				}
			}
			return linenum;
		},
		
		getXLocationAtXCoordInLine:function(xcoord,linenum) {
			var lines = this.getOptions('_lines');
			var fontname = this.getOptions('fontname');
			var fontsize = this.getOptions('fontsize');
			var text = this.getOptions('text');
			var wrapstyle = this.getOptions('wrapstyle');
			var align = this.getOptions('align');
			var size = this.getOptions('size');
			var textareasize = this.getOptions('_textareasize');
			
			var line = lines[linenum];
				
			var cache = MishaTools_Canvas_Cache.GetFreeCache(false);
			var ctx = cache.context;
			ctx.font = fontsize + 'px ' + fontname;
			
			if(align == 'right') xcoord -= Math.max(textareasize.x - Math.floor(ctx.measureText(line).width));
			else if(align == 'center') xcoord -= Math.max(Math.floor((textareasize.x - ctx.measureText(line).width)/2),0);
			
			var xlocation = undefined;
			
			if(xcoord < 0) xlocation = 0;
			else {
				for(var i=0;i<line.length;i++) {
					var leftbound = ctx.measureText(line.substring(0,i)).width;
					var width = ctx.measureText(line[i]).width;
					
					if(xcoord <= leftbound + width/2 && xcoord >= leftbound) {
						xlocation = i;
					} else if(xcoord > leftbound + width/2 && xcoord <= leftbound + width) {
						xlocation = i+1;
					}
					
					if(!Object.isUndefined(xlocation)) break;
				}
				
				if(Object.isUndefined(xlocation)) xlocation = line.length;
			}
			
			if(xlocation == 0) {
				var xpos = this.getPositionAtLocation($C(xlocation,linenum));
				
				if(xpos > 0 && ((text[xpos-1] != ' ' && wrapstyle == 'layout') || wrapstyle != 'layout') && text[xpos-1] != '\n') xlocation++;
			}
			
			cache.release();
			
			return xlocation;
		},
		
		getLocationAtPosition:function(position) {
			
			var text = this.getOptions('text');
			var lines = this.getOptions('_lines');
			var wrapstyle = this.getOptions('wrapstyle');
			
			var previouslengths = 0;
			for(var i=0; i<lines.length; i++) {
				var linelength = lines[i].length;
				if(position <= previouslengths + linelength) return $C(position - previouslengths,i);
				previouslengths += linelength;
				if((text[previouslengths] == ' ' && wrapstyle == 'layout') || text[previouslengths] == '\n') previouslengths++;
			}
			
			return undefined;
		},
		
		getPositionAtLocation:function(location) {
			var text = this.getOptions('text');
			var lines = this.getOptions('_lines');
			var wrapstyle = this.getOptions('wrapstyle');
			
			var position = 0;
			for(var i=0;i<location.y;i++) {
				position += lines[i].length;
				if((text[position] == ' ' && wrapstyle == 'layout') || text[position] == '\n') position++;
			}
			position += location.x;
			return position;
		},
		
		getCoordAtLocation:function(location,visible) {
			var fontsize = this.getOptions('fontsize');
			var fontname = this.getOptions('fontname');
			var lines = this.getOptions('_lines');
			var lineoffset = this.getOptions('lineoffset');
			var renderlines = this.getOptions('_renderlines');
			var size = this.getOptions('size');
			var align = this.getOptions('align');
			var textareasize = this.getOptions('_textareasize');
			
			//make sure it is visible
			if(visible && (location.y < lineoffset || location.y > lineoffset+renderlines.length-1)) return undefined;
			
			var cache = MishaTools_Canvas_Cache.GetFreeCache(false);
			var ctx = cache.context;
			ctx.font = fontsize + 'px ' + fontname;
			var line = lines[location.y];
			var coord = $C(Math.round(ctx.measureText(line.substring(0,location.x)).width),(location.y-lineoffset) * fontsize);
			
			if(align == 'right') coord = coord.add($C(Math.max(textareasize.x - Math.floor(ctx.measureText(line).width),0),0));
			else if(align == 'center') coord = coord.add($C(Math.max(Math.floor((textareasize.x - ctx.measureText(line).width)/2),0),0));
			
			cache.release();
			return coord;
		},
		
		getLocationAtCoord:function(coord,visible) {
			var lineoffset = this.getOptions('lineoffset');
			var renderlines = this.getOptions('_renderlines');
			var lines = this.getOptions('_lines');
			var linenum = this.getLineAtYCoord(coord.y);
			var text = this.getOptions('text');
			var size = this.getOptions('size');
			var align = this.getOptions('align');
			
			//make sure it is visible
			if(visible && (linenum < lineoffset || coord.y <= 0)) return $C(0,lineoffset);
			else if(visible && linenum > lineoffset+renderlines.length-1) return $C(lines[lineoffset+renderlines.length-1].length,lineoffset+renderlines.length-1);
			
			if(!Object.isUndefined(linenum)) {
				return $C(this.getXLocationAtXCoordInLine(coord.x,linenum),linenum);
			} else return this.getLocationAtPosition(text.length);
		},
		
		getCoordAtPosition:function(position,visible) {
			return this.getCoordAtLocation(this.getLocationAtPosition(position),visible);
		},
		
		getPositionAtCoord:function(coord,visible) {
			return this.getPositionAtLocation(this.getLocationAtCoord(coord),visible);
		},
		
		tab:function($super) {
			$super();
			this.selectAll();
		}
	});
	
	Object.extend(MishaTools_Canvas_Rect_Text_Interactive,{
		ActivateTextShim:function(position,type) {
			var dummy = $('_dummytextinput');
			
			if(!dummy)	{
				dummy = new Element('input',{type:type,id:'_dummytextinput',style:'opacity:0; position: absolute; z-index: -400; top:' + position.y + 'px; left:' + position.x + 'px; -moz-user-select: none; -webkit-user-select: none; -ms-user-select: none;',tabindex:-1});
				document.body.appendChild(dummy);
			} else {
				dummy.setStyle({top:position.y + 'px',left:position.x + 'px'});
				
				if(dummy.readAttribute('type') != type) {
					dummy.writeAttribute('type',type);
				}
			}
			
			dummy.focus();
		}
	});
});